/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package org.wso2.registry.config;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.wso2.registry.RegistryException;
import org.wso2.registry.secure.HSQLDBInitializer;
import org.wso2.registry.jdbc.hsql.DBUtils;
import org.wso2.registry.jdbc.utils.RegistryDataSource;
import org.wso2.registry.i18n.Messages;

import javax.servlet.ServletConfig;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.sql.Connection;
import java.sql.SQLException;

public class RegistryConfiguration {

    private DataBaseConfiguration defaultDataBaseConfiguration = null;
    private Map dbconfgigMap = new HashMap();

    /**
     * This method will read the registry.xml and using the data given in the file registry
     * configuration will be populated. And if the xmlLocation is null then it will use the one in
     * the class path.
     *
     * @param xmlLocation Location of registry.xml
     * @param servletConfig the active servlet configuration
     * @throws org.wso2.registry.RegistryException
     *          : if something went wrong
     */
    public void populateRegistryConfig(String xmlLocation, ServletConfig servletConfig)
            throws RegistryException {
        InputStream in;
        if (xmlLocation != null) {
            try {
                in = new FileInputStream(xmlLocation);
            } catch (FileNotFoundException e) {
                throw new RegistryException(e.getMessage());
            }
        } else {
            in = servletConfig.getServletContext().getResourceAsStream("/WEB-INF/registry.xml");
            if (in == null) {
                in = Thread.currentThread().getContextClassLoader().getResourceAsStream(
                        "org/wso2/registry/servlet/registry.xml");
            }
            if (in == null) {
                throw new RegistryException(Messages.getMessage("inputstream.null",
                        "org/wso2/servlet/registry.xml"));
            }
        }
        try {
            StAXOMBuilder buillder = new StAXOMBuilder(in);
            OMElement configElement = buillder.getDocumentElement();
            if (configElement != null) {

                Iterator dbconfgs = configElement.getChildrenWithName(new QName("dbconfig"));
                while (dbconfgs.hasNext()) {
                    OMElement dbconfig = (OMElement)dbconfgs.next();
                    DataBaseConfiguration dataBaseConfiguration = new DataBaseConfiguration();

                    String dbName = dbconfig.getAttributeValue(new QName("name"));
                    if (dbName == null) {
                        throw new RegistryException(Messages.getMessage("dbname.null"));
                    }
                    dataBaseConfiguration.setConfigName(dbName);

                    if (dbName.equals("sample")) {

                        dataBaseConfiguration.setDbUrl(getDBURL(servletConfig));
                        dataBaseConfiguration.setDriverName("org.hsqldb.jdbcDriver");
                        dataBaseConfiguration.setUserName("sa");
                        dataBaseConfiguration.setPassWord("");

                        // create the in-process database

                        RegistryDataSource dataSource = null;
                        try {
                            dataSource = new RegistryDataSource(
                                    dataBaseConfiguration.getDbUrl(),
                                    dataBaseConfiguration.getDriverName(),
                                    dataBaseConfiguration.getUserName(),
                                    dataBaseConfiguration.getPassWord());

                            DBUtils dbUtils = new DBUtils();
                            Connection conn = dataSource.getConnection();
                            dbUtils.initializeDatabase(conn);
                            conn.close();

                            HSQLDBInitializer dbInitializer = new HSQLDBInitializer();
                            dbInitializer.createHSQLTables(dataSource);

                        } catch (SQLException e) {
                            String msg = "Create sample in-process database.";
                            throw new RegistryException(msg);
                        }

                    } else {

                        OMElement dataSource = dbconfig.getFirstChildWithName(new QName("dataSource"));
                        if (dataSource != null) {
                            dataBaseConfiguration.setDataSourceName(dataSource.getText());

                        } else {

                            OMElement userName = dbconfig.getFirstChildWithName(new QName("userName"));
                            if (userName != null) {
                                dataBaseConfiguration.setUserName(userName.getText());
                            }

                            OMElement password = dbconfig.getFirstChildWithName(new QName("password"));
                            if (password != null) {
                                dataBaseConfiguration.setPassWord(password.getText());
                            }

                            OMElement url = dbconfig.getFirstChildWithName(new QName("url"));
                            if (url != null) {
                                dataBaseConfiguration.setDbUrl(url.getText());
                            }

                            OMElement driverName =
                                    dbconfig.getFirstChildWithName(new QName("driverName"));
                            if (driverName != null) {
                                dataBaseConfiguration.setDriverName(driverName.getText());
                            }
                        }
                    }

                    dbconfgigMap.put(dbName, dataBaseConfiguration);
                }

                OMElement currentConfigElement =
                        configElement.getFirstChildWithName(new QName("currentConfig"));
                if (currentConfigElement == null) {
                    String msg = "Current database configuration is not defined.";
                    throw new RegistryException(msg);
                }

                String currentConfigName = currentConfigElement.getText();
                DataBaseConfiguration currentConfiguration =
                        getDataBaseConfiguration(currentConfigName);
                setDefaultDataBaseConfiguration(currentConfiguration);
            }

        } catch (XMLStreamException e) {
            throw new RegistryException(e.getMessage());
        }
    }

    public void addDataBaseConfiguration(DataBaseConfiguration dataBaseConfiguration) {
        dbconfgigMap.put(dataBaseConfiguration.getConfigName(), dataBaseConfiguration);
    }

    public DataBaseConfiguration getDataBaseConfiguration(String configName) {
        return (DataBaseConfiguration)dbconfgigMap.get(configName);
    }

    public void removeDataBaseConfiguration(String configName) {
        dbconfgigMap.remove(configName);
    }

    public OMElement getRegistryConfigAsXML() {

        OMFactory factory = OMAbstractFactory.getOMFactory();
        OMElement root = factory.createOMElement("wso2regsitry", null);

        if (getDefaultDataBaseConfiguration() != null) {
            OMElement currrentConfigElement = factory.createOMElement("currentConfig", null);
            currrentConfigElement.setText(getDefaultDataBaseConfiguration().getConfigName());
            root.addChild(currrentConfigElement);
        }

        Iterator values = dbconfgigMap.values().iterator();
        while (values.hasNext()) {
            DataBaseConfiguration dataBaseConfiguration = (DataBaseConfiguration)values.next();
            OMElement config = factory.createOMElement("dbconfig", null);
            OMElement url = factory.createOMElement("url", null);
            url.setText(dataBaseConfiguration.getDbUrl());
            config.addChild(url);
            OMElement userName = factory.createOMElement("userName", null);
            userName.setText(dataBaseConfiguration.getUserName());
            config.addChild(userName);
            OMElement password = factory.createOMElement("password", null);
            password.setText(dataBaseConfiguration.getPassWord());
            config.addChild(password);
            OMElement driverName = factory.createOMElement("driverName", null);
            driverName.setText(dataBaseConfiguration.getDriverName());
            config.addChild(driverName);
            config.addAttribute("name", dataBaseConfiguration.getConfigName(), null);
            root.addChild(config);
        }
        return root;
    }

    public DataBaseConfiguration getDefaultDataBaseConfiguration() {
        return defaultDataBaseConfiguration;
    }

    public void setDefaultDataBaseConfiguration(DataBaseConfiguration deafultDataBaseConfiguration) {
        this.defaultDataBaseConfiguration = deafultDataBaseConfiguration;
    }

    public Map getDbconfgigMap() {
        return dbconfgigMap;
    }

    private String getDBURL(ServletConfig config) {

        String webinf = config.getServletContext().getRealPath("/WEB-INF");
        String dbURL = "jdbc:hsqldb:" + webinf + "/sample/sample-db;shutdown=true";
        return dbURL;
    }
}
