/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.exceptions.ResourceNotFoundException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.jdbc.dao.VersionedResourceDAO;
import org.wso2.registry.jdbc.mediatypes.builtin.DefaultMediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.builtin.SQLQueryMediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.builtin.SynapseRepositoryMediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.builtin.WSDLMediaTypeHandler;
import org.wso2.usermanager.Realm;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;

/**
 * Handles the actions by delegating them to registered media type handlers. If a mediatype handler
 * is not registered for a media type, its handling will be delegated to the
 * DefaultMediaTypeHandler. This class initializes all media type handlers at start up so that they
 * can be looked up by the media type string.
 */
public class MediaTypeManager {

    private static final Log log = LogFactory.getLog(JDBCRegistry.class);

    private DataSource dataSource = null;

    private VersionedResourceDAO resourceDAO = new VersionedResourceDAO();

    private Map mediaTypeHandlers = new HashMap();
    private DefaultMediaTypeHandler defaultMediaTypeHandler;

    public MediaTypeManager(DataSource dataSource, Realm realm) {

        this.dataSource = dataSource;

        defaultMediaTypeHandler = new DefaultMediaTypeHandler(dataSource, realm, this);
        mediaTypeHandlers.put(RegistryConstants.DEFAULT_MEDIA_TYPE, defaultMediaTypeHandler);

        MediaTypeHandler sqlMediaTypeHandler =
                new SQLQueryMediaTypeHandler(dataSource, realm, this);
        mediaTypeHandlers.put(RegistryConstants.SQL_QUERY_MEDIA_TYPE, sqlMediaTypeHandler);

        MediaTypeHandler synRepoMediaTypeHandler =
                new SynapseRepositoryMediaTypeHandler(dataSource, realm, this);
        mediaTypeHandlers.put(
                RegistryConstants.SYNPASE_REPOSITORY_MEDIA_TYPE, synRepoMediaTypeHandler);

        MediaTypeHandler wsdlMediaTypeHandler =
                new WSDLMediaTypeHandler(dataSource, realm, this);
        mediaTypeHandlers.put(RegistryConstants.WSDL_MEDIA_TYPE, wsdlMediaTypeHandler);
    }

    public Resource get(String path) throws RegistryException {

        Resource resource = defaultMediaTypeHandler.get(path, null);

        Resource processedArtifact = null;

        if (resource == null) {
            throw new ResourceNotFoundException(path);
        }

        if (resource.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler)mediaTypeHandlers.get(resource.getMediaType());

            if (mediaTypeHandler != null) {
                processedArtifact = mediaTypeHandler.get(path, resource);
            }
        }

        if (processedArtifact != null) {
            return processedArtifact;
        } else {
            return resource;
        }
    }

    public void put(String path, Resource resource) throws RegistryException {

        // if a parent collection exists and it has an associated media type handler, invoke the
        // putChild() of it.

        String parentPath = getParentPath(path);
        if (parentPath != null) {
            Resource parentCollection = defaultMediaTypeHandler.get(parentPath, null);

            if (parentCollection != null && parentCollection.getMediaType() != null) {
                MediaTypeHandler parentMediaTypeHandler =
                        (MediaTypeHandler)mediaTypeHandlers.get(parentCollection.getMediaType());
                if (parentMediaTypeHandler != null) {
                    boolean allowed = parentMediaTypeHandler.putChild(path, resource);
                    if (!allowed) {
                        String msg = "Attempted to add unrecognized resource in to the typed " +
                                     "collection " + parentCollection.getMediaType();
                        log.info(msg);
                        throw new RegistryException(msg);
                    }
                }
            }
        }

        if (resource.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler)mediaTypeHandlers.get(resource.getMediaType());

            if (mediaTypeHandler != null) {
                if (!mediaTypeHandler.put(path, resource)) {
                    defaultMediaTypeHandler.put(path, resource);
                }
            } else {
                defaultMediaTypeHandler.put(path, resource);
            }
        } else {
            defaultMediaTypeHandler.put(path, resource);
        }
    }

    public void delete(String path) throws RegistryException {

        Resource resource = getRawArtifact(path);

        if (resource == null) {
            throw new RegistryException("Could not delete a non-existing resource.");
        }

        if (resource.getMediaType() != null) {
            MediaTypeHandler mediaTypeHandler =
                    (MediaTypeHandler)mediaTypeHandlers.get(resource.getMediaType());

            if (mediaTypeHandler != null) {
                if (!mediaTypeHandler.delete(path)) {
                    defaultMediaTypeHandler.delete(path);
                }
            } else {
                defaultMediaTypeHandler.delete(path);
            }
        } else {
            defaultMediaTypeHandler.delete(path);
        }
    }

    public MediaTypeHandler getMediaTypeHandler(String mediaType) {
        return (MediaTypeHandler)mediaTypeHandlers.get(mediaType);
    }

    public DefaultMediaTypeHandler getDefaultMediaTypeHandler() {
        return defaultMediaTypeHandler;
    }

    private Resource getRawArtifact(String path) throws RegistryException {

        Resource resource;

        Connection conn = null;
        try {
            conn = dataSource.getConnection();
        } catch (SQLException e) {
            throw new RegistryException(e.getMessage());
        }

        try {
            resource = resourceDAO.getLatestVersion(path, conn);

        } catch (SQLException e) {

            String msg = Messages.getMessage("resource.get.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return resource;
    }

    private String getParentPath(String resourcePath) {

        String parentPath;
        if (resourcePath.equals(RegistryConstants.ROOT_PATH)) {
            parentPath = null;
        } else {
            String formattedPath = resourcePath;
            if (resourcePath.endsWith(RegistryConstants.PATH_SEPARATOR)) {
                formattedPath = resourcePath.substring(
                        0, resourcePath.length() - RegistryConstants.PATH_SEPARATOR.length());
            }

            if (formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR) == 0) {
                parentPath = RegistryConstants.ROOT_PATH;
            } else {
                parentPath = formattedPath.substring(
                        0, formattedPath.lastIndexOf(RegistryConstants.PATH_SEPARATOR));
            }
        }

        return parentPath;
    }
}
