/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.web.actions;

import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.Resource;
import org.wso2.registry.secure.AuthorizationFailedException;
import org.wso2.registry.web.actions.utils.AdvancedResourceQuery;
import org.wso2.registry.web.actions.utils.ResourceData;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

public class AdvancedSearchAction extends AbstractRegistryAction {

    private String resourcePath;
    private String authorName;
    private String updaterName;
    private String createdAfter;
    private String createdBefore;
    private String updatedAfter;
    private String updatedBefore;
    private String tags;
    private String commentWords;
    private String content;
    private String resultType = "Resources";

    private List resourceDataList = new ArrayList();

    public void execute(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        Registry registry = getRegistry();

        if (resultType.equals("Resources")) {

            AdvancedResourceQuery query = new AdvancedResourceQuery();
            query.setResourceName(resourcePath);
            query.setAuthorName(authorName);
            query.setUpdaterName(updaterName);
            query.setCreatedAfter(computeDate(createdAfter));
            query.setCreatedBefore(computeDate(createdBefore));
            query.setUpdatedAfter(computeDate(updatedAfter));
            query.setUpdatedBefore(computeDate(updatedBefore));
            query.setCommentWords(commentWords);
            query.setTags(tags);

            Resource qResults = query.execute(registry);

            String[] childPaths = (String[]) qResults.getContent();
            for (int i = 0; i < childPaths.length; i++) {

                ResourceData resourceData = new ResourceData();
                resourceData.setResourcePath(childPaths[i]);

                if (childPaths[i] != null) {
                    if (RegistryConstants.ROOT_PATH.equals(childPaths[i])) {
                        resourceData.setName("root");
                    } else {
                        String[] parts = childPaths[i].split(RegistryConstants.PATH_SEPARATOR);
                        resourceData.setName(parts[parts.length - 1]);
                    }
                }

                try {
                    Resource child = registry.get(childPaths[i]);

                    resourceData.setResourceType(
                            child.isDirectory()? ResourceData.COLLECTION : ResourceData.RESOURCE);
                    resourceData.setAuthorUserName(child.getAuthorUserName());
                    resourceData.setDescription(child.getDescription());
                    resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                    resourceData.setCreatedOn(child.getCreatedTime());
                    calculateAverageStars(resourceData);

                    resourceDataList.add(resourceData);

                } catch (AuthorizationFailedException e) {
                    // do not show unauthorized resource in search results.
                    continue;
                }
            }
        }

    }

    public void executeDeprecated(HttpServletRequest request) throws RegistryException {

        setRequest(request);

        // contructing sqls is prefered here, as otherwise we have to create and store very large
        // number of query resources to support combined criterias.

        boolean queryStarted = false;

        String pathQuery = "";
        if (resourcePath != null && resourcePath.length() != 0) {
            pathQuery = " WHERE R.PATH='" + resourcePath + "'";
            queryStarted = true;
        }

        String authorQuery = "";
        if (authorName != null && authorName.length() != 0) {
            if (queryStarted) {
                authorQuery = " AND R.AUTHOR='" + authorName + "'";
            } else {
                authorQuery = " WHERE R.AUTHOR='" + authorName + "'";
                queryStarted = true;
            }
        }

        //String sql = "SELECT A.AID FROM ARTIFACTS A, VERSIONS V, TAGS T, COMMENTS C WHERE A.AID=V.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE A.AID=V2.AID) AND A.AID=T.AID AND A.AID=C.AID AND A.PATH='c1200/r4'";
        String fromVersions = "";
        String versionQuery = "";
        if (updaterName != null && updaterName.length() != 0) {
            fromVersions = ", VERSIONS V";
            if (queryStarted) {
                versionQuery = " AND V.AID=R.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE R.AID=V2.AID) AND V.AUTHOR='" + updaterName + "'";
            } else {
                versionQuery = " WHERE V.AID=R.AID AND V.VN IN (SELECT MAX(V2.VN) FROM VERSIONS V2 WHERE R.AID=V2.AID) AND V.AUTHOR='" + updaterName + "'";
            }
        }

        String fromComments = "";
        String commentsQuery = "";
        if (commentWords != null && commentWords.length() != 0) {
            fromComments = ", COMMENTS C";
            if (queryStarted) {
                commentsQuery = " AND C.AID=R.AID AND C.COMMENT_TEXT LIKE '" + commentWords + "'";
            } else {
                commentsQuery = " WHERE C.AID=R.AID AND C.COMMENT_TEXT LIKE '%" + commentWords + "%'";
                queryStarted = true;
            }
        }

        String fromTags = "";
        String tagsQuery = "";
        if (tags != null && tags.length() != 0) {
            String[] tag = tags.split(" ");
            if (tag.length > 0) {
                fromTags = ", TAGS T";

                // todo: complete with OR tags
                if (queryStarted) {
                    tagsQuery = " AND T.AID=R.AID AND";
                } else {
                    tagsQuery = " WHERE T.AID=R.AID AND";
                    queryStarted = true;
                }

                tagsQuery = tagsQuery + " (T.TAG_NAME='" + tag[0] + "'";
                for (int i = 1; i < tag.length; i++) {
                    tagsQuery = tagsQuery + " OR T.TAG_NAME='" + tag[i] + "'";
                }
                tagsQuery = tagsQuery + ")";
            }
        }

        String sql = "SELECT R.PATH FROM ARTIFACTS R" + fromVersions + fromComments + fromTags + pathQuery + authorQuery + versionQuery + commentsQuery + tagsQuery;

        Registry registry = getRegistry();
        Resource q1 = new Resource();
        q1.setContent(sql);
        q1.setMediaType(RegistryConstants.SQL_QUERY_MEDIA_TYPE);
        q1.setProperty(RegistryConstants.RESULT_TYPE_PROPERTY_NAME,
                RegistryConstants.RESOURCES_RESULT_TYPE);
        String qPath = "/queries/advanced/" + getUserName() + "/q1";
        registry.put(qPath, q1);

        Resource qResults = registry.executeQuery(qPath, new String[] {} );

        String[] childPaths = (String[]) qResults.getContent();
        for (int i = 0; i < childPaths.length; i++) {

            ResourceData resourceData = new ResourceData();
            resourceData.setResourcePath(childPaths[i]);

            if (childPaths[i] != null) {
                if (RegistryConstants.ROOT_PATH.equals(childPaths[i])) {
                    resourceData.setName("root");
                } else {
                    String[] parts = childPaths[i].split(RegistryConstants.PATH_SEPARATOR);
                    resourceData.setName(parts[parts.length - 1]);
                }
            }

            try {
                Resource child = registry.get(childPaths[i]);

                resourceData.setResourceType(
                        child.isDirectory()? ResourceData.COLLECTION : ResourceData.RESOURCE);
                resourceData.setAuthorUserName(child.getAuthorUserName());
                resourceData.setDescription(child.getDescription());
                resourceData.setAverageRating(registry.getAverageRating(child.getPath()));
                resourceData.setCreatedOn(child.getCreatedTime());
                calculateAverageStars(resourceData);
                
                resourceDataList.add(resourceData);

            } catch (AuthorizationFailedException e) {
                continue;
            }


        }
    }

    public String getResourcePath() {
        return resourcePath;
    }

    public void setResourcePath(String resourcePath) {
        this.resourcePath = resourcePath;
    }

    public String getAuthorName() {
        return authorName;
    }

    public void setAuthorName(String authorName) {
        this.authorName = authorName;
    }

    public String getUpdaterName() {
        return updaterName;
    }

    public void setUpdaterName(String updaterName) {
        this.updaterName = updaterName;
    }

    public String getCreatedAfter() {
        return createdAfter;
    }

    public void setCreatedAfter(String createdAfter) {
        this.createdAfter = createdAfter;
    }

    public String getCreatedBefore() {
        return createdBefore;
    }

    public void setCreatedBefore(String createdBefore) {
        this.createdBefore = createdBefore;
    }

    public String getUpdatedAfter() {
        return updatedAfter;
    }

    public void setUpdatedAfter(String updatedAfter) {
        this.updatedAfter = updatedAfter;
    }

    public String getUpdatedBefore() {
        return updatedBefore;
    }

    public void setUpdatedBefore(String updatedBefore) {
        this.updatedBefore = updatedBefore;
    }

    public String getTags() {
        return tags;
    }

    public void setTags(String tags) {
        this.tags = tags;
    }

    public String getCommentWords() {
        return commentWords;
    }

    public void setCommentWords(String commentWords) {
        this.commentWords = commentWords;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public String getResultType() {
        return resultType;
    }

    public void setResultType(String resultType) {
        this.resultType = resultType;
    }

    public List getResourceDataList() {
        return resourceDataList;
    }

    public void setResourceDataList(List resourceDataList) {
        this.resourceDataList = resourceDataList;
    }

    /**
     * Converts given strings to Dates
     *
     * @param dateString Allowed formats
     * year/month/day:hour/minute/second
     * year/month/day
     *
     * @return Date corresponding to the given string date
     */
    private Date computeDate(String dateString) {

        if (dateString == null || dateString.length() == 0) {
            return null;
        }

        if (dateString.indexOf(":") > 0) {
            String[] dateTime = dateString.split(":");
            if (dateTime.length == 2) {
                String[] dateParts = dateTime[0].split("/");
                String[] timeParts = dateTime[1].split("/");
                Calendar calendar = new GregorianCalendar(
                        Integer.parseInt(dateParts[0]),
                        Integer.parseInt(dateParts[1]) - 1, // month value is zero based
                        Integer.parseInt(dateParts[2]),
                        Integer.parseInt(timeParts[0]),
                        Integer.parseInt(timeParts[1]),
                        Integer.parseInt(timeParts[2]));

                return calendar.getTime();
            }
        } else {

            String[] dateParts = dateString.split("/");
            Calendar calendar = new GregorianCalendar(
                    Integer.parseInt(dateParts[0]),
                    Integer.parseInt(dateParts[1]),
                    Integer.parseInt(dateParts[2]));

            return calendar.getTime();
        }
        
        return null;
    }

    private void calculateAverageStars(ResourceData resourceData) {

        float tempRating = resourceData.getAverageRating() * 1000;
        tempRating = Math.round(tempRating);
        tempRating = tempRating / 1000;
        resourceData.setAverageRating(tempRating);

        float averageRating = resourceData.getAverageRating();
        String[] averageStars = new String[5];

        for (int i = 0; i < 5; i++) {

            if (averageRating >= i + 1) {
                averageStars[i] = "04";

            } else if (averageRating <= i) {
                averageStars[i] = "00";

            } else {

                float fraction = averageRating - i;

                if (fraction <= 0.125) {
                    averageStars[i] = "00";

                } else if (fraction > 0.125 && fraction <= 0.375) {
                    averageStars[i] = "01";

                } else if (fraction > 0.375 && fraction <= 0.625) {
                    averageStars[i] = "02";

                } else if (fraction > 0.625 && fraction <= 0.875) {
                    averageStars[i] = "03";

                } else {
                    averageStars[i] = "04";

                }
            }
        }

        resourceData.setAverageStars(averageStars);
    }
}
